source("lib.r")

print("##############################################")
print("This script lists the implementation of SPCL, and a toy example (completely wrong prior knowledge) in our paper.")
print("##############################################")


###################################################
######## completely wrong prior knowledge #########
###################################################

#0) load data
id = letters[1:6]
myloss = c(0.1,0.2,0.4,0.6,0.5,0.3)
print("####### input loss ######")
print(myloss)


#1) optimize v without curriculum constraints
{
	v0 = replicate(length(id),0)		#initial values
	tolerance = 10^-7					#a small constant for optmization accuracy
	lambda = 0.83333					#parameter in self-paced learning							
	
	u2 = diag(replicate(length(v0),1))		#v >= 0 - 10^-7
	u3 = -1*diag(replicate(length(v0),1))	#-v >= -1 - 10^-7 i.e. v <= 1 + 10^-7
	ui = rbind(u2,u3)
	
	c2 = replicate(length(v0),-1*tolerance)	 #v >= 0 - 10^-7
	c3 = -1*replicate(length(v0),1+tolerance) #-v >= -1 - 10^-7 i.e. v <= 1 + 10^-7
	ci = c(c2,c3)
	
	
	ui %*% v0 - ci >= 0		#check the feasibility of the initial values v0

	#linear soft scheme
	solution1 = constrOptim(theta = v0, 
		  f = eval_f_hard,
		  grad = grad_f_hard,
		  ui = ui,
		  ci = ci,
		  loss = myloss, lambda=lambda)$par
		  
	#log soft scheme
	solution2 = constrOptim(theta = v0, 
		  f = eval_f_log,
		  grad = grad_f_log,
		  ui = ui,
		  ci = ci,
		  loss = myloss, lambda=lambda)$par
	
	print("##############################################")
	print("--hard scheme w/o curriculum Constraint")
	print(solution1)
	print("--log scheme w/o curriculum Constraint")
	print(solution2)
	
	
	
	#compare with the closed-form solution for unconstrained problems
	closed_solution1 = closedform_hard(myloss,lambda)
	closed_solution2 = closedform_log(myloss,lambda)
	
	print("##############################################")
	print(paste("--linear scheme MSE with closed-form solution", sum((solution1-closed_solution1)^2)))
	print(paste("--log scheme MSE with closed-form solution", sum((solution2-closed_solution2)^2)))
	
	print("##############################################")
	#print the curriculum (ordered by loss)
	print("SPL curriculum (ordered by loss)")
	print(id[getcurriculum_hard(solution1, myloss)])
}



#4) calculate curriculum constraints
A = matrix(0, nrow=length(id), ncol=1)
A[,1] = c(2.3, 2.2, 2.1, 2, 1.7, 1.5)			#curriculum constraints matrix
#A[,1] = c(0, 0, 0, 0, 0, 0)					#curriculum constraints matrix
c = c(6)										#curriculum constraints vector

print("####### A matrix ######")
print(A)


#5) optimize v with modality constraint (A)
{
	v0 = replicate(length(id),0)		#initial values
	tolerance = 10^-7					#a small constant for optmization accuracy
	lambda=0.8333						#parameter in self-paced learning

	u1 = -1*t(A)								#-Av >= -c	i.e.	Av <= c
	u2 = diag(replicate(length(v0),1))			#v >= 0 - 10^-7
	u3 = -1*diag(replicate(length(v0),1))		#-v >= -1 - 10^-7 i.e. v <= 1 + 10^-7
	ui = rbind(u1,u2,u3)						
	
	
	c1 = -1*c - tolerance						#-Av >= -c	i.e.	Av <= c
	c2 = replicate(length(v0),-1*tolerance)	 	#v >= 0 - 10^-7
	c3 = -1*replicate(length(v0),1+tolerance) 	#-v >= -1 - 10^-7 i.e. v <= 1 + 10^-7
	ci = c(c1,c2,c3)				
	
	#check the feasibility of initial values
	ui %*% v0 - ci >= 0
	
	solution4 = constrOptim(theta = v0, 
		f = eval_f_hard,
		grad = grad_f_hard,
		ui = ui,
		ci = ci,
		loss = myloss, lambda=lambda)$par

	#ui %*% solution3 - ci >= 0
	
	print("--hard scheme w/ curriculum Constraint")
	print(solution4)
	print("SPCL curriculum:")
	print(id[sort(solution4,index.return=TRUE, decreasing=TRUE)$ix])
	#print(id[getcurriculum_hard(solution4, myloss)])
	
	
	#print(ui %*% solution4 - ci >= 0)
}



######################################
########   Rank Correlation  #########
######################################
gt = c("a","b","c","d","e","f")
cl = c("f","e","d","c","b","a")
spl = id[getcurriculum_hard(solution1, myloss)]
spld = id[sort(solution4,index.return=TRUE, decreasing=TRUE)$ix]

gtrank = unlist(lapply(gt, function(x){which(gt==x)}))
clrank = unlist(lapply(cl, function(x){which(gt==x)}))
splrank = unlist(lapply(spl, function(x){which(gt==x)}))
spldrank = unlist(lapply(spld, function(x){which(gt==x)}))

print(paste("CL rank correlation =", cor(cbind(clrank,gtrank), method="kendall", use="pairwise")[1,2]))
print(paste("SPL rank correlation =", cor(cbind(splrank,gtrank), method="kendall", use="pairwise")[1,2]))
print(paste("SPCL rank correlation =", cor(cbind(spldrank,gtrank), method="kendall", use="pairwise")[1,2]))
